package arzumify.presence.matchmaker;

import arzumify.presence.maths.Vec2i;
import arzumify.presence.presences.Presence;
import arzumify.presence.presences.PresenceProvider;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public class MatchMaker<P extends Presence<P>, T extends PresenceProvider<P>> {
    public MatchMaker() {
    }

    private final Map<Channel, Map<Vec2i, Set<T>>> channels = new ConcurrentHashMap<>();

    public void Add(T match, Channel channel) {
        channels.putIfAbsent(channel, new ConcurrentHashMap<>());
        for (Vec2i chunk : match.presence().GetChunks()) {
            channels.get(channel).putIfAbsent(chunk, new HashSet<>());
            channels.get(channel).get(chunk).add(match);
        }
    }

    public Set<T> Search(T match, Channel channel) {
        Set<T> matches = new HashSet<>();
        Set<Vec2i> presenceChunks = match.presence().GetCollisionChunks();
        if (!Collections.disjoint(presenceChunks, channels.get(channel).keySet())) {
            presenceChunks.retainAll(channels.get(channel).keySet());
            for (Vec2i chunk : presenceChunks) {
                for (T matching : channels.get(channel).get(chunk)) {
                    if (match != matching) {
                        if (match.presence().Matches(matching.presence())) {
                            matches.add(matching);
                        }
                    }
                }
            }
        }
        return matches;
    }

    public void Remove(T match, Channel channel) {
        for (Vec2i chunk : match.presence().GetChunks()) {
            channels.get(channel).get(chunk).remove(match);
        }
    }
}
